@@ -15,6 +15,18 @@ from shep.error import (
 
 re_name = r'^[a-zA-Z_\.]+$'
 
+def join_elements(states):
+    return '_' + '__'.join(states)
+
+
+def split_elements(s):
+    if len(s) == 0:
+        return []
+    if s[0] == '_':
+        s = s[1:]
+    return s.split('__')
+
+
 class State:
     """State is an in-memory bitmasked state store for key-value pairs, or even just keys alone.
 
@@ -30,16 +42,26 @@ class State:
 
     base_state_name = 'NEW'
 
-    def __init__(self, bits, logger=None, verifier=None, check_alias=True, event_callback=None):
+    def __init__(self, bits, logger=None, verifier=None, check_alias=True, event_callback=None, default_state=None):
         self.__initial_bits = bits
         self.__bits = bits
         self.__limit = (1 << bits) - 1
         self.__c = 0
-        setattr(self, self.base_state_name, 0)
 
-        self.__reverse = {0: getattr(self, self.base_state_name)}
-        self.__keys = {getattr(self, self.base_state_name): []}
         self.__keys_reverse = {}
+
+        if default_state == None:
+            default_state = self.base_state_name
+        else:
+            default_state = self.__check_name_valid(default_state)
+            self.base_state_name = default_state
+            self.__keys_reverse[default_state] = 0
+
+        setattr(self, default_state, 0)
+
+        self.__reverse = {0: default_state}
+        self.__keys = {0: []}
+
         self.__contents = {}
         self.modified_last = {}
         self.verifier = verifier
@@ -183,6 +205,7 @@ class State:
         k = self.__check_name(k)
         v = self.__check_value(v)
         self.__set(k, v)
+        return v
 
 
     def to_name(self, k):
@@ -198,6 +221,7 @@ class State:
             v = self.__check_limit(v | a, pure=False)
         if self.is_pure(v):
             raise ValueError('use add to add pure values')
+        k = k.replace('.', '__')
         return self.__set(k, v)
 
 
@@ -217,16 +241,31 @@ class State:
         return self.__alias(k, *args)    
 
 
-    def all(self, pure=False, numeric=False):
+    def __all_bit(self):
+        r = []
+        r.append(self.name(0))
+        v = 1
+        while v < self.__limit:
+            r.append(self.name(v))
+            v <<= 1
+        return r
+
+
+    def all(self, pure=False, numeric=False, ignore_auto=True, bit_order=False):
         """Return list of all unique atomic and alias state strings.
         
         :rtype: list of ints
         :return: states
         """
         l = []
-        for k in dir(self):
+        v = None
+        if bit_order:
+            v = self.__all_bit()
+        else:
+            v = dir(self)
+        for k in v:
             state = None
-            if k[0] == '_':
+            if k[0] == '_' and ignore_auto:
                 continue
             if k.upper() != k:
                 continue
@@ -240,7 +279,8 @@ class State:
                 l.append(state)
             else:
                 l.append(k)
-        l.sort()
+        if not bit_order:
+            l.sort()
         return l
 
 
@@ -250,7 +290,7 @@ class State:
             return self.base_state_name
         c = 1
         for i in range(self.__bits):
-            if v & c > 0:
+            if (v & c) > 0:
                 if numeric:
                     r.append(c)
                 else:
@@ -260,15 +300,29 @@ class State:
         if numeric or not as_string:
             return r
 
-        return '_' + '.'.join(r)
+        if len(r) == 1:
+            return self.name(v)
+
+        return join_elements(r) #'_' + '.'.join(r)
 
 
-    def from_elements(self, k):
+    def from_elements(self, k, create_missing=False):
         r = 0
         if k[0] != '_':
             raise ValueError('elements string must start with underscore (_), got {}'.format(k))
-        for v in k[1:].split('.'):
-            r |= self.from_name(v) 
+        for v in k[1:].split('__'):
+            state = None
+            try:
+                state = self.from_name(v) 
+            except AttributeError as e:
+                pass
+
+            if state == None:
+                if not create_missing: 
+                    raise StateInvalid(v)
+                state = self.add(v)
+
+            r |= state
         return r
 
 
@@ -303,6 +357,8 @@ class State:
         :return: Numeric state value
         """
         k = self.__check_name_valid(k)
+        if k == self.base_state_name:
+            return 0
         return getattr(self, k)
 
 
@@ -410,16 +466,19 @@ class State:
             raise StateCorruptionError(to_state)
 
         if self.verifier != None:
-            r = self.verifier(self, from_state, to_state)
+            r = self.verifier(self, key, from_state, to_state)
             if r != None:
                 raise StateTransitionInvalid(r)
 
-        current_state_list.pop(idx) 
-
+        old_state = self.__keys_reverse.get(key)
         if self.event_callback != None:
-            old_state = self.__keys_reverse.get(key)
             self.event_callback(key, self.name(old_state), self.name(to_state))
 
+        if old_state == 0:
+            current_state_list.pop(idx)
+        else:
+            for k in self.elements(from_state, numeric=True):
+                self.__keys[k].remove(key)
         self.__add_state_list(to_state, key)
 
         self.register_modify(key)
@@ -458,12 +517,14 @@ class State:
     def unset(self, key, not_state, allow_base=False):
         """Unset a single bit, moving to a pure or alias state.
         
-        The resulting state cannot be State.base_state_name (0).
+        If allow_base is set to False, The resulting state cannot be State.base_state_name (0).
          
         :param key: Content key to modify state for
         :type key: str
         :param or_state: Atomic stat to add
         :type or_state: int
+        :paran allow_base: Allow state to be reset to 0
+        :type allow_base: bool
         :raises ValueError: State is not a single bit state, or attempts to revert to State.base_state_name
         :raises StateItemNotFound: Content key is not registered
         :raises StateInvalid: Resulting state after addition of atomic state is unknown
@@ -553,7 +614,7 @@ class State:
             return []
 
 
-    def sync(self, state=None):
+    def sync(self, state=None, not_state=None, ignore_auto=True):
         """Noop method for interface implementation providing sync to backend.
         
         :param state: State to sync.
@@ -661,3 +722,7 @@ class State:
             del self.modified_last[key]
         except KeyError:
             pass
+
+
+    def count(self):
+        return self.__c
